@@ -113,6 +113,9 @@ class CloudFilesConnection(OpenStackBaseConnection):
         self.accept_format = 'application/json'
         self.cdn_request = False
 
+        if self._ex_force_service_region:
+            self.service_region = self._ex_force_service_region
+
     def get_endpoint(self):
         # First, we parse out both files and cdn endpoints
         # for each auth version
@@ -131,6 +134,13 @@ class CloudFilesConnection(OpenStackBaseConnection):
         if self.cdn_request:
             eps = cdn_eps
 
+        if self.service_region:
+            _eps = []
+            for ep in eps:
+                if ep['region'].lower() == self.service_region.lower():
+                    _eps.append(ep)
+            eps = _eps
+
         if len(eps) == 0:
             raise LibcloudError('Could not find specified endpoint')
 
@@ -234,7 +244,8 @@ class CloudFilesStorageDriver(StorageDriver, OpenStackDriverMixin):
         return LazyList(get_more=self._get_more, value_dict=value_dict)
 
     def get_container(self, container_name):
-        response = self.connection.request('/%s' % (container_name),
+        container_name_encoded = self._encode_container_name(container_name)
+        response = self.connection.request('/%s' % (container_name_encoded),
                                            method='HEAD')
 
         if response.status == httplib.NO_CONTENT:
@@ -248,8 +259,11 @@ class CloudFilesStorageDriver(StorageDriver, OpenStackDriverMixin):
 
     def get_object(self, container_name, object_name):
         container = self.get_container(container_name)
-        response = self.connection.request('/%s/%s' % (container_name,
-                                                       object_name),
+        container_name_encoded = self._encode_container_name(container_name)
+        object_name_encoded = self._encode_container_name(object_name)
+
+        response = self.connection.request('/%s/%s' % (container_name_encoded,
+                                                       object_name_encoded),
                                            method='HEAD')
         if response.status in [httplib.OK, httplib.NO_CONTENT]:
             obj = self._headers_to_object(
@@ -301,9 +315,9 @@ class CloudFilesStorageDriver(StorageDriver, OpenStackDriverMixin):
         return response.status in [httplib.CREATED, httplib.ACCEPTED]
 
     def create_container(self, container_name):
-        container_name = self._clean_container_name(container_name)
+        container_name_encoded = self._encode_container_name(container_name)
         response = self.connection.request(
-            '/%s' % (container_name), method='PUT')
+            '/%s' % (container_name_encoded), method='PUT')
 
         if response.status == httplib.CREATED:
             # Accepted mean that container is not yet created but it will be
@@ -320,7 +334,7 @@ class CloudFilesStorageDriver(StorageDriver, OpenStackDriverMixin):
         raise LibcloudError('Unexpected status code: %s' % (response.status))
 
     def delete_container(self, container):
-        name = self._clean_container_name(container.name)
+        name = self._encode_container_name(container.name)
 
         # Only empty container can be deleted
         response = self.connection.request('/%s' % (name), method='DELETE')
@@ -395,8 +409,8 @@ class CloudFilesStorageDriver(StorageDriver, OpenStackDriverMixin):
                                 extra=extra, iterator=iterator)
 
     def delete_object(self, obj):
-        container_name = self._clean_container_name(obj.container.name)
-        object_name = self._clean_object_name(obj.name)
+        container_name = self._encode_container_name(obj.container.name)
+        object_name = self._encode_object_name(obj.name)
 
         response = self.connection.request(
             '/%s/%s' % (container_name, object_name), method='DELETE')
@@ -409,6 +423,26 @@ class CloudFilesStorageDriver(StorageDriver, OpenStackDriverMixin):
 
         raise LibcloudError('Unexpected status code: %s' % (response.status))
 
+    def ex_purge_object_from_cdn(self, obj, email=None):
+        """
+        Purge edge cache for the specified object.
+
+        @param email: Email where a notification will be sent when the job
+        completes. (optional)
+        @type email: C{str}
+        """
+        container_name = self._encode_container_name(obj.container.name)
+        object_name = self._encode_object_name(obj.name)
+        headers = {'X-Purge-Email': email} if email else {}
+
+        response = self.connection.request('/%s/%s' % (container_name,
+                                                       object_name),
+                                           method='DELETE',
+                                           headers=headers,
+                                           cdn_request=True)
+
+        return response.status == httplib.NO_CONTENT
+
     def ex_get_meta_data(self):
         """
         Get meta data
@@ -584,14 +618,14 @@ class CloudFilesStorageDriver(StorageDriver, OpenStackDriverMixin):
         extra = extra or {}
         meta_data = extra.get('meta_data')
 
-        container_name_cleaned = self._clean_container_name(container.name)
-        object_name_cleaned = self._clean_object_name(object_name)
-        request_path = '/%s/%s' % (container_name_cleaned, object_name_cleaned)
+        container_name_encoded = self._encode_container_name(container.name)
+        object_name_encoded = self._encode_object_name(object_name)
+        request_path = '/%s/%s' % (container_name_encoded, object_name_encoded)
 
         headers = {'X-Auth-Token': self.connection.auth_token,
                    'X-Object-Manifest': '%s/%s/' %
-                                        (container_name_cleaned,
-                                         object_name_cleaned)}
+                                        (container_name_encoded,
+                                         object_name_encoded)}
 
         data = ''
         response = self.connection.request(request_path,
@@ -647,8 +681,8 @@ class CloudFilesStorageDriver(StorageDriver, OpenStackDriverMixin):
                     upload_func_kwargs, extra=None, file_path=None,
                     iterator=None, verify_hash=True):
         extra = extra or {}
-        container_name_cleaned = self._clean_container_name(container.name)
-        object_name_cleaned = self._clean_object_name(object_name)
+        container_name_encoded = self._encode_container_name(container.name)
+        object_name_encoded = self._encode_object_name(object_name)
         content_type = extra.get('content_type', None)
         meta_data = extra.get('meta_data', None)
 
@@ -658,7 +692,7 @@ class CloudFilesStorageDriver(StorageDriver, OpenStackDriverMixin):
                 key = 'X-Object-Meta-%s' % (key)
                 headers[key] = value
 
-        request_path = '/%s/%s' % (container_name_cleaned, object_name_cleaned)
+        request_path = '/%s/%s' % (container_name_encoded, object_name_encoded)
         result_dict = self._upload_object(
             object_name=object_name, content_type=content_type,
             upload_func=upload_func, upload_func_kwargs=upload_func_kwargs,
@@ -692,9 +726,9 @@ class CloudFilesStorageDriver(StorageDriver, OpenStackDriverMixin):
             raise LibcloudError('status_code=%s' % (response.status),
                                 driver=self)
 
-    def _clean_container_name(self, name):
+    def _encode_container_name(self, name):
         """
-        Clean container name.
+        Encode container name so it can be used as part of the HTTP request.
         """
         if name.startswith('/'):
             name = name[1:]
@@ -712,7 +746,7 @@ class CloudFilesStorageDriver(StorageDriver, OpenStackDriverMixin):
 
         return name
 
-    def _clean_object_name(self, name):
+    def _encode_object_name(self, name):
         name = urlquote(name)
         return name
 
